@@ -10,9 +10,9 @@ require Exporter;
 require DynaLoader;
 @ISA = qw(Exporter DynaLoader);
 
-@EXPORT_OK = qw(utf16 utf8 utf7 ucs2 ucs4 latin1 uchr uhex);
+@EXPORT_OK = qw(utf16 utf8 utf7 ucs2 ucs4 latin1 uchr uhex byteswap2 byteswap4);
 
-$VERSION = '2.03'; # $Id: String.pm,v 1.23 1999/07/28 06:46:39 gisle Exp $
+$VERSION = '2.05'; # $Id: String.pm,v 1.26 1999/11/17 13:30:51 gisle Exp $
 
 $UTF7_OPTIONAL_DIRECT_CHARS ||= 1;
 
@@ -85,12 +85,12 @@ sub _dump_arg
 sub concat
 {
     #_dump_arg("concat", @_);
-    my($self, $other) = @_;
+    my($self, $other, $reversed) = @_;
     my $class = ref($self);
     unless (UNIVERSAL::isa($other, 'Unicode::String')) {
 	$other = Unicode::String->new($other);
     }
-    my $str = $$self . $$other;
+    my $str = $reversed ? $$other . $$self : $$self . $$other;
     bless \$str, $class;
 }
 
@@ -297,6 +297,12 @@ sub length
     int(length($$self) / 2);
 }
 
+sub byteswap
+{
+   my $self = shift;
+   byteswap2($$self);
+   $self;
+}
 
 sub unpack
 {
@@ -695,6 +701,19 @@ operation is overloaded as the assignment operator.
 Returns the length of the I<Unicode::String>.  Surrogate pairs are
 still counted as 2.
 
+=item $us->byteswap;
+
+This method will swap the bytes in the internal representation of the
+I<Unicode::String> object.
+
+Unicode reserve the character U+FEFF character as a byte order mark.
+This works because the swapped character, U+FFFE, is reserved to not
+be valid.  For strings that have the byte order mark as the first
+character, we can guaranty to get the byte order right with the
+following code:
+
+   $ustr->byteswap if $ustr->ord == 0xFFFE;
+
 =item $us->unpack;
 
 Returns a list of integers each representing an UTF-16 character code.
@@ -741,6 +760,29 @@ I<Unicode::String> object).
 
 =back
 
+=head1 FUNCTIONS
+
+The following utility functions are provided.  They will be exported
+on request.
+
+=over 4
+
+=item byteswap2($str, ...)
+
+This function will swap 2 and 2 bytes in the strings passed as
+arguments.  This can be used to fix up UTF-16 or UCS-2 strings from
+litle-endian systems.  If this function is called in void context,
+then it will modify its arguments in-place.  Otherwise, then swapped
+strings are returned.
+
+=item byteswap4($str, ...)
+
+The byteswap4 function works similar to byteswap2, but will reverse
+the order of 4 and 4 bytes.  Can be used to fix litle-endian UCS-4
+strings.
+
+=back
+
 =head1 SEE ALSO
 
 L<Unicode::CharName>,
